/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SGraphyItem } from "@saga-web/graphy/";
import { SMouseEvent } from "@saga-web/base/";;
import {
    SColor,
    SLineCapStyle,
    SPainter,
    SPath2D,
    SPoint,
    SPolygonUtil,
    SRect,
    SLine
} from "@saga-web/draw";
import { PolygonData } from "./type/PolygonData";
import { Point } from "./type/Point";
/**
 * 编辑多边形
 *
 * @author  韩耀龙
 */
export class SPolygonItem extends SGraphyItem {
    /** X坐标最小值  */
    private minX = Number.MAX_SAFE_INTEGER;
    /** X坐标最大值  */
    private maxX = Number.MIN_SAFE_INTEGER;
    /** Y坐标最小值  */
    private minY = Number.MAX_SAFE_INTEGER;
    /** Y坐标最大值  */
    private maxY = Number.MIN_SAFE_INTEGER;
    /** 轮廓线坐标  */
    pointList: SPoint[] = [];
    /** 是否闭合    */
    closeFlag: boolean = false;
    // 当前状态 1：show 2 创建中，3 编辑中
    _status: number = 2;
    // 获取当前状态
    get getStatus(): number {
        return this._status;
    }
    // 编辑当前状态
    set setStatus(value: number) {
        this._status = value;
        this.update();
    };
    data: PolygonData | null = null;
    /** 边框颜色 */
    borderColor: SColor = new SColor("#0091FF");
    /** 填充颜色 */
    brushColor: SColor = new SColor("#1EE887");
    /** border宽 只可输入像素宽*/
    borderLine: number = 4;
    /** 鼠标移动点  */
    private lastPoint = new SPoint();

    /**
    * 构造函数
    *
    * @param parent    指向父对象
    * @param data      PolygonData数据
    */

    constructor(parent: SGraphyItem | null, data: PolygonData | null) {
        super(parent);
        this.data = data
        if (data && data.PointList) {
            this._xyzListToSPointList(data.PointList);
        };
        this.zOrder = 100;
    }


    /**
    * 储存新的多边形顶点
    * @param x   点位得x坐标
    * @param y   点位得y坐标
    * @param i   储存所在索引
    * @return SPoint
    */

    reservePoint(x: number, y: number, i?: number): SPoint {
        const point = new SPoint(x, y);
        if (typeof i == 'undefined') {
            this.pointList.push(point)
        } else {
            this.pointList.splice(i, 0, point);
        }
        this.update()
        return point
    }


    /**
    * 删除点位
    * @param i   删除点所在的索引
    * @return SPoint
    */

    deletePoint(i?: number): SPoint | null | undefined {
        let point = null;
        if (typeof i != 'undefined') {
            if (this.pointList.length - 1 >= i) {
                point = this.pointList[i];
                this.pointList.splice(i, 0);
            } else {
                point = null
            }
        } else {
            point = this.pointList.pop();
        }
        return point
    }


    /**
    * 多边形顶点的移动位置
    * @param x   点位得x坐标
    * @param y   点位得y坐标
    * @param i   点位得i坐标
    * @return SPoint
    */

    movePoint(x: number, y: number, i: number, ): SPoint | null | undefined {
        let point = null;
        if (this.pointList.length) {
            this.pointList[i].x = x;
            this.pointList[i].y = y;
        }
        point = this.pointList[i];
        return point
    }


    /**
    * 打印出多边形数组
    * @return SPoint[]
    */

    PrintPointList(): SPoint[] {
        return this.pointList
    }

    /**
     * Item对象边界区域
     *
     * @return SRect
     */
    boundingRect(): SRect {
        return new SRect(
            this.minX,
            this.minY,
            this.maxX - this.minX,
            this.maxY - this.minY
        );
    } // Function boundingRect()


    /**
    * xyz数据转换为SPoint格式函数；获取外接矩阵参数
    *
    * @param arr    外层传入的数据PointList
    */

    protected _xyzListToSPointList(arr: Point[]) {
        if (arr.length) {
            this.pointList = arr.map(it => {
                let x = it.X,
                    y = it.Y;
                if (x < this.minX) {
                    this.minX = x;
                }
                if (y < this.minY) {
                    this.minY = y;
                }
                if (x > this.maxX) {
                    this.maxX = x;
                }
                if (y > this.maxY) {
                    this.maxY = y;
                }
                return new SPoint(x, y);
            });
        } else {
            this.pointList = []
        }
    }


    /**
    * 构造函数  展示多边形轮廓
    *
    */

    protected showPolygon(painter: SPainter, pointList: SPoint[]) {
        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = this.borderColor;
        painter.pen.lineWidth = painter.toPx(this.borderLine);
        painter.brush.color = this.brushColor;
        painter.drawPolygon([...this.pointList]);
    }


    /**
    * 构造函数  创建多边形
    *
    */

    protected createPolygon(painter: SPainter, pointList: SPoint[]) {
        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = new SColor("#0091FF");
        painter.pen.lineWidth = painter.toPx(4);
        painter.drawPolyline(this.pointList);
        if (this.lastPoint) {
            painter.drawLine(
                pointList[this.pointList.length - 1],
                this.lastPoint
            );
        };
        painter.pen.color = SColor.Transparent;
        painter.pen.lineWidth = painter.toPx(4);
        painter.drawPolygon([...this.pointList, this.lastPoint]);
    }


    /**
* 构造函数  创建多边形
*
*/
    protected editPolygon(painter: SPainter, pointList: SPoint[], point: SPoint) {
        // 展示多边形
        this.showPolygon(painter, pointList);
        // 绘制顶点块


    }





    ////////////以下为鼠标事件

    /**
    * 鼠标双击事件
    *
    * @param	event         事件参数
    * @return	boolean
    */

    onDoubleClick(event: SMouseEvent): boolean {
        return true;
    } // Function onDoubleClick()

    /**
 * 键盘事件
 *
 * @param	event         事件参数
 * @return	boolean
 */
    onKeyDown(event: KeyboardEvent): boolean {
        if (this._status == 2 && event.code == 'Enter') {
            this._status = 1
        }
        this.update()
        return true;
    } // Function onKeyDown()
    /**
     * 鼠标双击事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseDown(event: SMouseEvent): boolean {
        if (this._status == 2) {
            this.reservePoint(event.x, event.y)
        }
        return true;
    } // Function onMouseDown()
    /**
     * 鼠标双击事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseEnter(event: SMouseEvent): boolean {
        return true;
    } // Function onMouseDown()
    /**
     * 鼠标双击事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseLeave(event: SMouseEvent): boolean {
        return true;
    } // Function onMouseDown()
    onMouseMove(event: SMouseEvent): boolean {
        if (this._status == 2) {
            this.lastPoint.x = event.x;
            this.lastPoint.y = event.y;
        }
        this.update()
        return true;
    } // Function onMouseDown()
    onMouseUp(event: SMouseEvent): boolean {
        return true;
    } // Function onMouseUp()
    onResize(event: SMouseEvent): boolean {
        return true;
    } // Function onMouseUp()


    ////////////////////////

    /**
* Item绘制操作
*
* @param   painter       painter对象
*/
    onDraw(painter: SPainter): void {
        // 当状态为展示状态
        if (this._status == 1) {
            // 闭合多边形
            this.showPolygon(painter, this.pointList);
        } else if (this._status == 2) {
            // 创建状态
            this.createPolygon(painter, this.pointList)
        } else {
            // 编辑状态
            // this.editPolygon(painter, this.pointList);
        }
    } // Function onDraw()
}
